class ResourceID:
    def __init__(self, APP_ID):
        self.ACTION_BAR_CONTAINER = f"{APP_ID}:id/action_bar_container"
        self.ACTION_BAR_LARGE_TITLE = f"{APP_ID}:id/action_bar_large_title"
        self.ACTION_BAR_NEW_TITLE_CONTAINER = (
            f"{APP_ID}:id/action_bar_new_title_container"
        )
        self.ACTION_BAR_OVERFLOW_ICON = f"{APP_ID}:id/action_bar_overflow_icon"
        self.ACTION_BAR_SEARCH_EDIT_TEXT = f"{APP_ID}:id/action_bar_search_edit_text"
        self.ACTION_BAR_TEXTVIEW_TITLE = f"{APP_ID}:id/action_bar_textview_title"
        self.ACTION_BAR_TITLE = f"{APP_ID}:id/action_bar_title"
        self.ACTION_BAR_TITLE_AUTO_SIZE = (
            f"{APP_ID}:id/action_bar_large_title_auto_size"
        )
        self.ACTION_ICON = f"{APP_ID}:id/action_icon"
        self.BOTTOM_SHEET_CONTAINER_VIEW = f"{APP_ID}:id/bottom_sheet_container_view"
        self.BUTTON = f"{APP_ID}:id/button"
        self.CAROUSEL_MEDIA_GROUP = f"{APP_ID}:id/carousel_media_group"
        self.CAROUSEL_INDEX_INDICATOR_TEXT_VIEW = (
            f"{APP_ID}:id/carousel_index_indicator_text_view"
        )
        self.COORDINATOR_ROOT_LAYOUT = f"{APP_ID}:id/coordinator_root_layout"
        self.DIALOG_ROOT_VIEW = f"{APP_ID}:id/dialog_root_view"
        self.DIALOG_CONTAINER = f"{APP_ID}:id/dialog_container"
        self.DIRECT_TEXT_MESSAGE_TEXT_VIEW = (
            f"{APP_ID}:id/direct_text_message_text_view"
        )
        self.ECHO_TEXT = f"{APP_ID}:id/echo_text"
        self.FEED_INLINE_COMPOSER_BUTTON_TEXTVIEW = (
            f"{APP_ID}:id/feed_inline_composer_button_textview"
        )
        self.FIND_PEOPLE_DISMISS_BUTTON = f"{APP_ID}:id/find_people_dismiss_button"
        self.FIXED_TABBAR_TABS_CONTAINER = f"{APP_ID}:id/fixed_tabbar_tabs_container"
        self.FOLLOW_LIST_CONTAINER = f"{APP_ID}:id/follow_list_container"
        self.FOLLOW_LIST_SORTING_OPTIONS_RECYCLER_VIEW = (
            f"{APP_ID}:id/follow_list_sorting_options_recycler_view"
        )
        self.FOLLOW_LIST_USERNAME = f"{APP_ID}:id/follow_list_username"
        self.FOLLOW_SHEET_UNFOLLOW_ROW = f"{APP_ID}:id/follow_sheet_unfollow_row"
        self.FOOTER_SPACE = f"{APP_ID}:id/footer_space"
        self.GAP_VIEW = f"{APP_ID}:id/gap_view"
        self.IGDS_HEADLINE_BODY = f"{APP_ID}:id/igds_headline_body"
        self.IGDS_HEADLINE_EMPHASIZED_HEADLINE = (
            f"{APP_ID}:id/igds_headline_emphasized_headline"
        )
        self.IMAGE_BUTTON = f"{APP_ID}:id/image_button"
        self.LANGUAGE_LIST_LOCALE = f"{APP_ID}:id/language_locale_list"
        self.LAYOUT_COMMENT_THREAD_EDITTEXT = (
            f"{APP_ID}:id/layout_comment_thread_edittext"
        )
        self.LAYOUT_COMMENT_THREAD_POST_BUTTON_CLICK_AREA = (
            f"{APP_ID}:id/layout_comment_thread_post_button_click_area"
        )
        self.LIST = "android:id/list"
        self.MEDIA_GROUP = f"{APP_ID}:id/media_group"
        self.MENU_SETTINGS_ROW = f"{APP_ID}:id/menu_settings_row"
        self.NEGATIVE_BUTTON = f"{APP_ID}:id/negative_button"
        self.MESSAGE_TEXT = f"{APP_ID}:id/message_text"
        self.NETEGO_CAROUSEL_HEADER = f"{APP_ID}:id/netego_carousel_header"
        self.NEW_FEED_PILL = f"{APP_ID}:id/new_feed_pill"
        self.PRIMARY_BUTTON = f"{APP_ID}:id/primary_button"
        self.PRIVATE_PROFILE_EMPTY_STATE = f"{APP_ID}:id/private_profile_empty_state"
        self.PROFILE_HEADER_BIO_TEXT = f"{APP_ID}:id/profile_header_bio_text"
        self.PROFILE_HEADER_BUSINESS_CATEGORY = (
            f"{APP_ID}:id/profile_header_business_category"
        )
        self.PROFILE_HEADER_FULL_NAME = f"{APP_ID}:id/profile_header_full_name"
        self.PROFILE_HEADER_AVATAR_CONTAINER_TOP_LEFT_STUB = (
            f"{APP_ID}:id/profile_header_avatar_container_top_left_stub"
        )
        self.PROFILE_TAB_LAYOUT = f"{APP_ID}:id/profile_tab_layout"
        self.PROFILE_TAB_ICON_VIEW = f"{APP_ID}:id/profile_tab_icon_view"
        self.PROFILE_TABS_CONTAINER = f"{APP_ID}:id/profile_tabs_container"
        self.REEL_RING = f"{APP_ID}:id/reel_ring"
        self.REEL_VIEWER_IMAGE_VIEW = f"{APP_ID}:id/reel_viewer_image_view"
        self.REEL_VIEWER_MEDIA_CONTAINER = f"{APP_ID}:id/reel_viewer_media_container"
        self.REEL_VIEWER_TIMESTAMP = f"{APP_ID}:id/reel_viewer_timestamp"
        self.REEL_VIEWER_TITLE = f"{APP_ID}:id/reel_viewer_title"
        self.ROW_COMMENT_SUB_ITEMS_BAR = f"{APP_ID}:id/row_comment_sub_items_bar"
        self.ROW_COMMENT_TEXTVIEW_COMMENT = f"{APP_ID}:id/row_comment_textview_comment"
        self.ROW_COMMENT_TEXTVIEW_TIME_AGO = (
            f"{APP_ID}:id/row_comment_textview_time_ago"
        )
        self.ROW_FEED_BUTTON_COMMENT = f"{APP_ID}:id/row_feed_button_comment"
        self.ROW_FEED_BUTTON_LIKE = f"{APP_ID}:id/row_feed_button_like"
        self.ROW_FEED_COMMENT_TEXTVIEW_LAYOUT = (
            f"{APP_ID}:id/row_feed_comment_textview_layout"
        )
        self.ROW_FEED_LIKE_COUNT_FACEPILE_STUB = (
            f"{APP_ID}:id/row_feed_like_count_facepile_stub"
        )
        self.ROW_FEED_PHOTO_PROFILE_NAME = f"{APP_ID}:id/row_feed_photo_profile_name"
        self.ROW_FEED_TEXTVIEW_LIKES = f"{APP_ID}:id/row_feed_textview_likes"
        self.ROW_FEED_VIEW_GROUP_BUTTONS = f"{APP_ID}:id/row_feed_view_group_buttons"
        self.ROW_HASHTAG_TEXTVIEW_TAG_NAME = (
            f"{APP_ID}:id/row_hashtag_textview_tag_name"
        )
        self.INFORM_BODY = f"{APP_ID}:id/inform_body"
        self.ROW_PLACE_TITLE = f"{APP_ID}:id/row_place_title"
        self.ROW_LOAD_MORE_BUTTON = f"{APP_ID}:id/row_load_more_button"
        self.ROW_PROFILE_HEADER_EMPTY_PROFILE_NOTICE_CONTAINER = (
            f"{APP_ID}:id/row_profile_header_empty_profile_notice_container"
        )
        self.ROW_PROFILE_HEADER_EMPTY_PROFILE_NOTICE_TITLE = (
            f"{APP_ID}:id/row_profile_header_empty_profile_notice_title"
        )
        self.ROW_PROFILE_HEADER_FOLLOWERS_CONTAINER = f"{APP_ID}:id/row_profile_header_followers_container|{APP_ID}:id/row_profile_header_container_followers"
        self.ROW_PROFILE_HEADER_FOLLOWING_CONTAINER = f"{APP_ID}:id/row_profile_header_following_container|{APP_ID}:id/row_profile_header_container_following"
        self.ROW_PROFILE_HEADER_IMAGEVIEW = f"{APP_ID}:id/row_profile_header_imageview"
        self.ROW_PROFILE_HEADER_TEXTVIEW_FOLLOWERS_COUNT = (
            f"{APP_ID}:id/row_profile_header_textview_followers_count"
        )
        self.ROW_PROFILE_HEADER_TEXTVIEW_POST_CONTAINER = (
            f"{APP_ID}:id/row_profile_header_post_count_container"
        )
        self.ROW_PROFILE_HEADER_TEXTVIEW_FOLLOWING_COUNT = (
            f"{APP_ID}:id/row_profile_header_textview_following_count"
        )
        self.ROW_PROFILE_HEADER_TEXTVIEW_POST_COUNT = (
            f"{APP_ID}:id/row_profile_header_textview_post_count"
        )
        self.ROW_SEARCH_EDIT_TEXT = f"{APP_ID}:id/row_search_edit_text"
        self.ROW_SEARCH_USER_USERNAME = f"{APP_ID}:id/row_search_user_username"
        self.ROW_SIMPLE_TEXT_TEXTVIEW = f"{APP_ID}:id/row_simple_text_textview"
        self.ROW_THREAD_COMPOSER_BUTTON_SEND = (
            f"{APP_ID}:id/row_thread_composer_button_send"
        )
        self.ROW_THREAD_COMPOSER_EDITTEXT = f"{APP_ID}:id/row_thread_composer_edittext"
        self.ROW_USER_CONTAINER_BASE = f"{APP_ID}:id/row_user_container_base"
        self.ROW_USER_PRIMARY_NAME = f"{APP_ID}:id/row_user_primary_name"
        self.ROW_USER_TEXTVIEW = f"{APP_ID}:id/row_user_textview"
        self.SEARCH = f"{APP_ID}:id/search"
        self.SECONDARY_LABEL = f"{APP_ID}:id/secondary_label"
        self.SEE_ALL_BUTTON = f"{APP_ID}:id/see_all_button"
        self.SORTING_ENTRY_ROW_ICON = f"{APP_ID}:id/sorting_entry_row_icon"
        self.SWIPE_REFRESH_ANIMATED_PROGRESSBAR_CONTAINER = (
            f"{APP_ID}:id/swipe_refresh_animated_progressbar_container"
        )
        self.TAB_AVATAR = f"{APP_ID}:id/tab_avatar"
        self.TAB_BAR = f"{APP_ID}:id/tab_bar"
        self.TAB_BUTTON_NAME_TEXT = f"{APP_ID}:id/tab_button_name_text"
        self.TAB_BUTTON_FALLBACK_ICON = f"{APP_ID}:id/tab_button_fallback_icon"
        self.TITLE_VIEW = f"{APP_ID}:id/title_view"
        self.UNIFIED_FOLLOW_LIST_TAB_LAYOUT = (
            f"{APP_ID}:id/unified_follow_list_tab_layout"
        )
        self.ZOOMABLE_VIEW_CONTAINER = f"{APP_ID}:id/zoomable_view_container"

        self.CAROUSEL_MEDIA_GROUP_AND_ZOOMABLE_VIEW_CONTAINER = (
            f"{self.ZOOMABLE_VIEW_CONTAINER}|{self.CAROUSEL_MEDIA_GROUP}"
        )
        self.GAP_VIEW_AND_FOOTER_SPACE = f"{self.GAP_VIEW}|{self.FOOTER_SPACE}"
        self.BLOCK_POPUP = f"{self.DIALOG_CONTAINER}|{self.DIALOG_ROOT_VIEW}"


class TabBarText:
    ACTIVITY_CONTENT_DESC = "Activity"
    EFFECTS_CONTENT_DESC = "Effects"
    HOME_CONTENT_DESC = "Home"
    IGTV_CONTENT_DESC = "IGTV"
    ORDERS_CONTENT_DESC = "Orders"
    PHOTOS_OF_YOU_CONTENT_DESC = "Photos of You"
    POSTS_CONTENT_DESC = "Grid View"
    PROFILE_CONTENT_DESC = "Profile"
    RECENT_CONTENT_DESC = "Recent"
    REELS_CONTENT_DESC = "Reels"
    SEARCH_CONTENT_DESC = "Search and Explore"


class ClassName:
    BUTTON = "android.widget.Button"
    BUTTON_OR_TEXTVIEW_REGEX = "android.widget.Button|android.widget.TextView"
    BUTTON_OR_FRAME_LAYOUT_REGEX = "android.widget.Button|android.widget.FrameLayout"
    EDIT_TEXT = "android.widget.EditText"
    FRAME_LAYOUT = "android.widget.FrameLayout"
    HORIZONTAL_SCROLL_VIEW = "android.widget.HorizontalScrollView"
    IMAGE_VIEW = "android.widget.ImageView"
    LIST_VIEW = "android.widget.ListView"
    LINEAR_LAYOUT = "android.widget.LinearLayout"
    RECYCLER_VIEW = "androidx.recyclerview.widget.RecyclerView"
    TEXT_VIEW = "android.widget.TextView"
    VIEW = "android.view.View"
    VIEW_GROUP = "android.view.ViewGroup"
    VIEW_PAGER = "androidx.viewpager.widget.ViewPager"
